@@ -1142,15 +1142,109 @@ int ScalarArray<T>::productQ(char side, char trans, ScalarArray<T>* c) const {
   // In qrDecomposition(), tau is stored in the last column of 'this'
   // it is not valid to work with 'tau' inside the array 'a' because zunmqr modifies 'a'
   // during computation. So we work on a copy of tau.
-  T tau[std::min(rows, cols)];
-  memcpy(tau, const_ptr(0, cols-1), sizeof(T)*std::min(rows, cols));
+  std::vector<T> tau(std::min(rows, cols));
+  memcpy(tau.data(), const_ptr(0, cols-1), sizeof(T)*std::min(rows, cols));
 
   // We don't use c->ptr() on purpose, because c->is_ortho is preserved here (Q is orthogonal)
-  int info = proxy_lapack_convenience::or_un_mqr(side, trans, c->rows, c->cols, cols, const_ptr(), lda, tau, c->m, c->lda);
+  int info = proxy_lapack_convenience::or_un_mqr(side, trans, c->rows, c->cols, cols, const_ptr(), lda, tau.data(), c->m, c->lda);
   HMAT_ASSERT(!info);
   return 0;
 }
 
+template <typename T>
+void ScalarArray<T>::reflect(Vector<T> &v_house, double beta, char transA)
+{
+    assert(abs(beta) >= abs(std::numeric_limits<T>::epsilon()));
+    ScalarArray<T> w_house(1,cols);
+    w_house.gemm(transA , 'N' , beta ,  &v_house ,this, 0);
+    rankOneUpdateT(1,v_house, w_house);
+}
+
+template <typename T> void ScalarArray<T>::cpqrDecomposition(int * &sigma, double * &tau ,int *rank, double epsilon)
+{ 
+  double normSqr=0;
+  int iter=0;
+  int min_dim=std::min(cols , rows);
+  sigma=(int*)malloc(min_dim*sizeof(int));
+  tau=(double*)malloc(min_dim*sizeof(double));
+  //sigma will be modifief each iteration to keep track of the columns transpositions
+  for (int i =0 ; i < cols ; i++)
+  {
+    sigma[i]=i;
+  }
+  char transA;
+  if(std::is_same<Z_t, T>::value || std::is_same<C_t, T>::value) transA='C';
+  else transA='T';
+  std::vector<double> normSqrCol(cols);
+  int pivot=0;
+  double max_norm=0;
+  for (int i = 0 ; i<cols ; i++)
+  {
+    Vector<T> col_i(*this , i);
+    normSqrCol[i]=col_i.normSqr();
+    if (max_norm < normSqrCol[i])
+    {
+      max_norm=normSqrCol[i];
+      pivot=i;
+    }
+    normSqr+=normSqrCol[i];
+  }
+  double norm_init=sqrt(normSqr);
+  while (sqrt(normSqr) > epsilon*norm_init && iter < min_dim)
+  {
+    T x1=get(iter , pivot); 
+    //Swap the columns, the coeficients of normSqrCol and update sigma
+    T *tmp=(T*)malloc(sizeof(T)*rows);
+    memcpy(tmp, &get(0,iter), sizeof(T)*rows);
+    memcpy (&get(0,iter), &get(0,pivot), sizeof(T)*rows);
+    memcpy (&get(0,pivot), tmp, sizeof(T)*rows);
+    free(tmp);
+    double tmp_d=normSqrCol[iter];
+    normSqrCol[iter]=normSqrCol[pivot];
+    normSqrCol[pivot]=tmp_d;
+    int tmpInt=sigma[iter];
+    sigma[iter]=sigma[pivot];
+    sigma[pivot]=tmpInt;
+
+    //Construction of Householder vector
+
+    ScalarArray<T> remainder(*this , iter , rows-iter , iter , cols-iter);
+    Vector<T> v_house(rows-iter);
+    T mu=sqrt(normSqrCol[iter]);
+    T alpha=std::abs(x1)!=0 ? x1+(x1/std::abs(x1))*mu : mu;
+    v_house[0]=std::abs(x1)!=0 ? 1 : 0;
+    for (int i = 1 ; i<rows-iter ; i++)
+    {
+      v_house[i]=remainder.get(i,0)/alpha;
+    }
+
+    //Householder update (only changes the bottom right part of m[iter : , iter :]=remainder)
+    double beta=(-2/v_house.normSqr());
+    tau[iter]=beta;//needed to later re-construct Q from this
+    ScalarArray <T> w_house(1,cols-iter);
+    w_house.gemm(transA , 'N' , beta ,&v_house , &remainder ,  0);
+    remainder.rankOneUpdateT(1 , v_house , w_house);
+
+    //norm udpate and determination of the next pivot
+
+    max_norm=0;
+    for(int i = 1 ; i < cols-iter ; i++)
+    {
+      normSqrCol[i+iter]-=std::pow(std::abs(remainder.get(0 , i)),2);
+      normSqr-=std::pow(std::abs(remainder.get(0 , i)),2);
+      if(normSqrCol[i]>max_norm)
+      {
+        max_norm=normSqrCol[i+iter];
+        pivot=i+iter;
+      }
+    }
+    normSqr-=std::pow(std::abs(remainder.get(0,0)),2);
+    memcpy(&m[(iter)*rows+iter+1], &v_house.m[1] , (rows-iter-1)*sizeof(T));//storing v_house in the low part of this
+    iter++;
+  }
+  *rank=iter;
+  tau=(double*)realloc(tau, sizeof(double)*iter);
+}
 template<typename T> int ScalarArray<T>::modifiedGramSchmidt(ScalarArray<T> *result, double prec, int initialPivot ) {
   DECLARE_CONTEXT;
   Timeline::Task t(Timeline::MGS, &rows, &cols, &initialPivot);
