@@ -89,7 +89,7 @@ template<typename T> class ScalarArray {
 
 private:
   /*! True if the matrix owns its memory, ie has to free it upon destruction */
-  char ownsMemory:1;
+  unsigned char ownsMemory:1;
 protected:
   /// Fortran style pointer (columnwise)
   T* m;
@@ -98,7 +98,7 @@ protected:
   int *is_ortho;
 #endif
   /*! True if we own 'is_ortho' (there are cases where we own the flag and not the memory, with the constructor taking a 'T*' as input) */
-  char ownsFlag:1;
+  unsigned char ownsFlag:1;
 public:
   /// Number of rows
   int rows;
@@ -434,6 +434,28 @@ public:
    */
   int productQ(char side, char trans, ScalarArray<T>* c) const;
 
+/**
+ this=(I+beta.v_house*v_house^t)*this
+ make the reflection with reflectors v_house on this 
+*/
+  void reflect(Vector<T> &v_house, double beta, char transA);
+/**
+ Pivoted QR decomposition.
+ with this filled as in LAPACK geqp3
+
+ H_rank*...H_1*this=[R11 | R12]
+                    [  0 | R22]
+  with R11 triangular
+       ||R22||< epsilon*||this||
+ 
+    \param sigma the inverse permutation of the columns (use later to reconstuct R)
+    \param tau such that H_k=(I+tau[k]v_house*v_house^t) (as in LAPACK geqp3 and geqrf)
+    \param rank number of reflections
+    \param epsilon maximal relative norm for the residual 
+*/
+  void cpqrDecomposition(int * &sigma, double  * &tau, int *rank,  double epsilon);
+
+  void cpqrBlock( double **tau , int *rank, double epsilon);
 
   /** @brief Wrapper for Lapack trmm with B=this */
   void trmm(Side side, Uplo uplo, char transA, Diag diag, T alpha, const ScalarArray<T> * a);
@@ -512,6 +534,8 @@ public:
     *is_ortho = flag;
     static char *test = getenv("HMAT_TEST_ORTHO");
     if (flag && test) assert(getOrtho() == testOrtho());
+#else
+    (void)flag; // unused
 #endif
   }
 
@@ -537,6 +561,7 @@ public:
   public:
     Vector(T* _m, int _rows):ScalarArray<T>(_m, _rows, 1){}
     Vector(int _rows):ScalarArray<T>(_rows, 1){}
+    Vector(int _rows, bool zeroinit):ScalarArray<T>(_rows , 1 , zeroinit){}
     /** \brief Create Vector with column 'col' of existing ScalarArray
      */
     Vector(const ScalarArray<T> &d, int _col):ScalarArray<T>(d, 0, d.rows, _col, 1){}
